@@ -1,64 +1,77 @@
 # ext/orderinglist.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-"""A custom list that manages index/position information for its children.
+"""A custom list that manages index/position information for contained
+elements.
 
 :author: Jason Kirtland
 
-``orderinglist`` is a helper for mutable ordered relationships.  It will intercept
-list operations performed on a relationship collection and automatically
-synchronize changes in list position with an attribute on the related objects.
-(See :ref:`advdatamapping_entitycollections` for more information on the general pattern.)
+``orderinglist`` is a helper for mutable ordered relationships.  It will
+intercept list operations performed on a :func:`.relationship`-managed
+collection and
+automatically synchronize changes in list position onto a target scalar
+attribute.
 
-Example: Two tables that store slides in a presentation.  Each slide
-has a number of bullet points, displayed in order by the 'position'
-column on the bullets table.  These bullets can be inserted and re-ordered
-by your end users, and you need to update the 'position' column of all
-affected rows when changes are made.
+Example: A ``slide`` table, where each row refers to zero or more entries
+in a related ``bullet`` table.   The bullets within a slide are
+displayed in order based on the value of the ``position`` column in the
+``bullet`` table.   As entries are reordered in memory, the value of the
+``position`` attribute should be updated to reflect the new sort order::
 
-.. sourcecode:: python+sql
 
-    slides_table = Table('Slides', metadata,
-                         Column('id', Integer, primary_key=True),
-                         Column('name', String))
+    Base = declarative_base()
 
-    bullets_table = Table('Bullets', metadata,
-                          Column('id', Integer, primary_key=True),
-                          Column('slide_id', Integer, ForeignKey('Slides.id')),
-                          Column('position', Integer),
-                          Column('text', String))
+    class Slide(Base):
+        __tablename__ = 'slide'
 
-     class Slide(object):
-         pass
-     class Bullet(object):
-         pass
+        id = Column(Integer, primary_key=True)
+        name = Column(String)
 
-     mapper(Slide, slides_table, properties={
-           'bullets': relationship(Bullet, order_by=[bullets_table.c.position])
-     })
-     mapper(Bullet, bullets_table)
+        bullets = relationship("Bullet", order_by="Bullet.position")
 
-The standard relationship mapping will produce a list-like attribute on each Slide
-containing all related Bullets, but coping with changes in ordering is totally
-your responsibility.  If you insert a Bullet into that list, there is no
-magic- it won't have a position attribute unless you assign it it one, and
-you'll need to manually renumber all the subsequent Bullets in the list to
-accommodate the insert.
+    class Bullet(Base):
+        __tablename__ = 'bullet'
+        id = Column(Integer, primary_key=True)
+        slide_id = Column(Integer, ForeignKey('slide.id'))
+        position = Column(Integer)
+        text = Column(String)
 
-An ``orderinglist`` can automate this and manage the 'position' attribute on all
-related bullets for you.
+The standard relationship mapping will produce a list-like attribute on each
+``Slide`` containing all related ``Bullet`` objects,
+but coping with changes in ordering is not handled automatically.
+When appending a ``Bullet`` into ``Slide.bullets``, the ``Bullet.position``
+attribute will remain unset until manually assigned.   When the ``Bullet``
+is inserted into the middle of the list, the following ``Bullet`` objects
+will also need to be renumbered.
 
-.. sourcecode:: python+sql
+The :class:`.OrderingList` object automates this task, managing the
+``position`` attribute on all ``Bullet`` objects in the collection.  It is
+constructed using the :func:`.ordering_list` factory::
 
-    mapper(Slide, slides_table, properties={
-           'bullets': relationship(Bullet,
-                               collection_class=ordering_list('position'),
-                               order_by=[bullets_table.c.position])
-    })
-    mapper(Bullet, bullets_table)
+    from sqlalchemy.ext.orderinglist import ordering_list
+
+    Base = declarative_base()
+
+    class Slide(Base):
+        __tablename__ = 'slide'
+
+        id = Column(Integer, primary_key=True)
+        name = Column(String)
+
+        bullets = relationship("Bullet", order_by="Bullet.position",
+                                collection_class=ordering_list('position'))
+
+    class Bullet(Base):
+        __tablename__ = 'bullet'
+        id = Column(Integer, primary_key=True)
+        slide_id = Column(Integer, ForeignKey('slide.id'))
+        position = Column(Integer)
+        text = Column(String)
+
+With the above mapping the ``Bullet.position`` attribute is managed::
 
     s = Slide()
     s.bullets.append(Bullet())
@@ -69,73 +82,87 @@ related bullets for you.
     s.bullets[2].position
     >>> 2
 
-Use the ``ordering_list`` function to set up the ``collection_class`` on relationships
-(as in the mapper example above).  This implementation depends on the list
-starting in the proper order, so be SURE to put an order_by on your relationship.
+The :class:`.OrderingList` construct only works with **changes** to a collection,
+and not the initial load from the database, and requires that the list be
+sorted when loaded.  Therefore, be sure to
+specify ``order_by`` on the :func:`.relationship` against the target ordering
+attribute, so that the ordering is correct when first loaded.
 
-.. warning:: 
+.. warning::
 
-  ``ordering_list`` only provides limited functionality when a primary
-  key column or unique column is the target of the sort.  Since changing the order of 
-  entries often means that two rows must trade values, this is not possible when 
-  the value is constrained by a primary key or unique constraint, since one of the rows
-  would temporarily have to point to a third available value so that the other row
-  could take its old value.   ``ordering_list`` doesn't do any of this for you, 
+  :class:`.OrderingList` only provides limited functionality when a primary
+  key column or unique column is the target of the sort.  Since changing the
+  order of entries often means that two rows must trade values, this is not
+  possible when the value is constrained by a primary key or unique
+  constraint, since one of the rows would temporarily have to point to a
+  third available value so that the other row could take its old
+  value. :class:`.OrderingList` doesn't do any of this for you,
   nor does SQLAlchemy itself.
 
-``ordering_list`` takes the name of the related object's ordering attribute as
+:func:`.ordering_list` takes the name of the related object's ordering attribute as
 an argument.  By default, the zero-based integer index of the object's
-position in the ``ordering_list`` is synchronized with the ordering attribute:
+position in the :func:`.ordering_list` is synchronized with the ordering attribute:
 index 0 will get position 0, index 1 position 1, etc.  To start numbering at 1
 or some other integer, provide ``count_from=1``.
 
-Ordering values are not limited to incrementing integers.  Almost any scheme
-can implemented by supplying a custom ``ordering_func`` that maps a Python list
-index to any value you require.
-
-
-
 
 """
-from sqlalchemy.orm.collections import collection
-from sqlalchemy import util
+from ..orm.collections import collection
+from .. import util
 
-__all__ = [ 'ordering_list' ]
+__all__ = ['ordering_list']
 
 
 def ordering_list(attr, count_from=None, **kw):
-    """Prepares an OrderingList factory for use in mapper definitions.
+    """Prepares an :class:`OrderingList` factory for use in mapper definitions.
 
-    Returns an object suitable for use as an argument to a Mapper relationship's
-    ``collection_class`` option.  Arguments are:
+    Returns an object suitable for use as an argument to a Mapper
+    relationship's ``collection_class`` option.  e.g.::
 
-    attr
+        from sqlalchemy.ext.orderinglist import ordering_list
+
+        class Slide(Base):
+            __tablename__ = 'slide'
+
+            id = Column(Integer, primary_key=True)
+            name = Column(String)
+
+            bullets = relationship("Bullet", order_by="Bullet.position",
+                                    collection_class=ordering_list('position'))
+
+    :param attr:
       Name of the mapped attribute to use for storage and retrieval of
       ordering information
 
-    count_from (optional)
+    :param count_from:
       Set up an integer-based ordering, starting at ``count_from``.  For
       example, ``ordering_list('pos', count_from=1)`` would create a 1-based
       list in SQL, storing the value in the 'pos' column.  Ignored if
       ``ordering_func`` is supplied.
 
-    Passes along any keyword arguments to ``OrderingList`` constructor.
+    Additional arguments are passed to the :class:`.OrderingList` constructor.
+
     """
 
     kw = _unsugar_count_from(count_from=count_from, **kw)
     return lambda: OrderingList(attr, **kw)
 
+
 # Ordering utility functions
+
+
 def count_from_0(index, collection):
     """Numbering function: consecutive integers starting at 0."""
 
     return index
 
+
 def count_from_1(index, collection):
     """Numbering function: consecutive integers starting at 1."""
 
     return index + 1
 
+
 def count_from_n_factory(start):
     """Numbering function: consecutive integers starting at arbitrary start."""
 
@@ -147,8 +174,9 @@ def count_from_n_factory(start):
         pass
     return f
 
+
 def _unsugar_count_from(**kw):
-    """Builds counting functions from keywrod arguments.
+    """Builds counting functions from keyword arguments.
 
     Keyword argument filter, prepares a simple ``ordering_func`` from a
     ``count_from`` argument, otherwise passes ``ordering_func`` on unchanged.
@@ -164,12 +192,13 @@ def _unsugar_count_from(**kw):
             kw['ordering_func'] = count_from_n_factory(count_from)
     return kw
 
+
 class OrderingList(list):
     """A custom list that manages position information for its children.
 
-    See the module and __init__ documentation for more details.  The
-    ``ordering_list`` factory function is used to configure ``OrderingList``
-    collections in ``mapper`` relationship definitions.
+    The :class:`.OrderingList` object is normally set up using the
+    :func:`.ordering_list` factory function, used in conjunction with
+    the :func:`.relationship` function.
 
     """
 
@@ -184,14 +213,14 @@ class OrderingList(list):
         This implementation relies on the list starting in the proper order,
         so be **sure** to put an ``order_by`` on your relationship.
 
-        ordering_attr
+        :param ordering_attr:
           Name of the attribute that stores the object's order in the
           relationship.
 
-        ordering_func
-          Optional.  A function that maps the position in the Python list to a
-          value to store in the ``ordering_attr``.  Values returned are
-          usually (but need not be!) integers.
+        :param ordering_func: Optional.  A function that maps the position in
+          the Python list to a value to store in the
+          ``ordering_attr``.  Values returned are usually (but need not be!)
+          integers.
 
           An ``ordering_func`` is called with two positional parameters: the
           index of the element in the list, and the list itself.
@@ -202,7 +231,7 @@ class OrderingList(list):
           like stepped numbering, alphabetical and Fibonacci numbering, see
           the unit tests.
 
-        reorder_on_append
+        :param reorder_on_append:
           Default False.  When appending an object with an existing (non-None)
           ordering value, that value will be left untouched unless
           ``reorder_on_append`` is true.  This is an optimization to avoid a
@@ -216,7 +245,7 @@ class OrderingList(list):
           making changes, any of whom happen to load this collection even in
           passing, all of the sessions would try to "clean up" the numbering
           in their commits, possibly causing all but one to fail with a
-          concurrent modification error.  Spooky action at a distance.
+          concurrent modification error.
 
           Recommend leaving this with the default of False, and just call
           ``reorder()`` if you're doing ``append()`` operations with
@@ -315,9 +344,24 @@ class OrderingList(list):
         self._reorder()
     # end Py2K
 
+    def __reduce__(self):
+        return _reconstitute, (self.__class__, self.__dict__, list(self))
+
     for func_name, func in locals().items():
         if (util.callable(func) and func.func_name == func_name and
             not func.__doc__ and hasattr(list, func_name)):
             func.__doc__ = getattr(list, func_name).__doc__
     del func_name, func
 
+
+def _reconstitute(cls, dict_, items):
+    """ Reconstitute an :class:`.OrderingList`.
+
+    This is the adjoint to :meth:`.OrderingList.__reduce__`.  It is used for
+    unpickling :class:`.OrderingList` objects.
+
+    """
+    obj = cls.__new__(cls)
+    obj.__dict__.update(dict_)
+    list.extend(obj, items)
+    return obj
