@@ -1,5 +1,5 @@
 # util/langhelpers.py
-# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -15,8 +15,63 @@ import re
 import sys
 import types
 import warnings
-from compat import update_wrapper, set_types, threading
-from sqlalchemy import exc
+from .compat import set_types, threading, \
+    callable, inspect_getfullargspec
+from functools import update_wrapper
+from .. import exc
+import hashlib
+from . import compat
+
+def md5_hex(x):
+    # Py3K
+    #x = x.encode('utf-8')
+    m = hashlib.md5()
+    m.update(x)
+    return m.hexdigest()
+
+class safe_reraise(object):
+    """Reraise an exception after invoking some
+    handler code.
+
+    Stores the existing exception info before
+    invoking so that it is maintained across a potential
+    coroutine context switch.
+
+    e.g.::
+
+        try:
+            sess.commit()
+        except:
+            with safe_reraise():
+                sess.rollback()
+
+    """
+
+    def __enter__(self):
+        self._exc_info = sys.exc_info()
+
+    def __exit__(self, type_, value, traceback):
+        # see #2703 for notes
+        if type_ is None:
+            exc_type, exc_value, exc_tb = self._exc_info
+            self._exc_info = None   # remove potential circular references
+            compat.reraise(exc_type, exc_value, exc_tb)
+        else:
+            self._exc_info = None   # remove potential circular references
+            compat.reraise(type_, value, traceback)
+
+def decode_slice(slc):
+    """decode a slice object as sent to __getitem__.
+
+    takes into account the 2.5 __index__() method, basically.
+
+    """
+    ret = []
+    for x in slc.start, slc.stop, slc.step:
+        if hasattr(x, '__index__'):
+            x = x.__index__()
+        ret.append(x)
+    return tuple(ret)
 
 def _unique_symbols(used, *bases):
     used = set(used)
@@ -32,13 +87,14 @@ def _unique_symbols(used, *bases):
         else:
             raise NameError("exhausted namespace for symbol base %s" % base)
 
+
 def decorator(target):
     """A signature-matching decorator factory."""
 
     def decorate(fn):
         if not inspect.isfunction(fn):
             raise Exception("not a decoratable function")
-        spec = inspect.getargspec(fn)
+        spec = inspect_getfullargspec(fn)
         names = tuple(spec[0]) + spec[1:3] + (fn.func_name,)
         targ_name, fn_name = _unique_symbols(names, 'target', 'fn')
 
@@ -47,14 +103,54 @@ def decorator(target):
 
         code = 'lambda %(args)s: %(target)s(%(fn)s, %(apply_kw)s)' % (
                 metadata)
-        decorated = eval(code, {targ_name:target, fn_name:fn})
+        decorated = eval(code, {targ_name: target, fn_name: fn})
         decorated.func_defaults = getattr(fn, 'im_func', fn).func_defaults
         return update_wrapper(decorated, fn)
     return update_wrapper(decorate, target)
 
 
+class PluginLoader(object):
 
-def get_cls_kwargs(cls):
+    def __init__(self, group, auto_fn=None):
+        self.group = group
+        self.impls = {}
+        self.auto_fn = auto_fn
+
+    def load(self, name):
+        if name in self.impls:
+            return self.impls[name]()
+
+        if self.auto_fn:
+            loader = self.auto_fn(name)
+            if loader:
+                self.impls[name] = loader
+                return loader()
+
+        try:
+            import pkg_resources
+        except ImportError:
+            pass
+        else:
+            for impl in pkg_resources.iter_entry_points(
+                                self.group, name):
+                self.impls[name] = impl.load
+                return impl.load()
+
+        from sqlalchemy import exc
+        raise exc.ArgumentError(
+                "Can't load plugin: %s:%s" %
+                (self.group, name))
+
+    def register(self, name, modulepath, objname):
+        def load():
+            mod = __import__(modulepath)
+            for token in modulepath.split(".")[1:]:
+                mod = getattr(mod, token)
+            return getattr(mod, objname)
+        self.impls[name] = load
+
+
+def get_cls_kwargs(cls, _set=None):
     """Return the full set of inherited kwargs for the given `cls`.
 
     Probes a class's __init__ method, collecting all named arguments.  If the
@@ -66,36 +162,35 @@ def get_cls_kwargs(cls):
     No anonymous tuple arguments please !
 
     """
+    toplevel = _set == None
+    if toplevel:
+        _set = set()
 
-    for c in cls.__mro__:
-        if '__init__' in c.__dict__:
-            stack = set([c])
-            break
-    else:
-        return []
+    ctr = cls.__dict__.get('__init__', False)
 
-    args = set()
-    while stack:
-        class_ = stack.pop()
-        ctr = class_.__dict__.get('__init__', False)
-        if (not ctr or
-            not isinstance(ctr, types.FunctionType) or
-            not isinstance(ctr.func_code, types.CodeType)):
-            stack.update(class_.__bases__)
-            continue
-
-        # this is shorthand for
-        # names, _, has_kw, _ = inspect.getargspec(ctr)
+    has_init = ctr and isinstance(ctr, types.FunctionType) and \
+        isinstance(ctr.func_code, types.CodeType)
 
+    if has_init:
         names, has_kw = inspect_func_args(ctr)
-        args.update(names)
-        if has_kw:
-            stack.update(class_.__bases__)
-    args.discard('self')
-    return args
+        _set.update(names)
+
+        if not has_kw and not toplevel:
+            return None
+
+    if not has_init or has_kw:
+        for c in cls.__bases__:
+            if get_cls_kwargs(c, _set) is None:
+                break
+
+    _set.discard('self')
+    return _set
+
+
 
 try:
     from inspect import CO_VARKEYWORDS
+
     def inspect_func_args(fn):
         co = fn.func_code
         nargs = co.co_argcount
@@ -103,11 +198,13 @@ try:
         args = list(names[:nargs])
         has_kw = bool(co.co_flags & CO_VARKEYWORDS)
         return args, has_kw
+
 except ImportError:
     def inspect_func_args(fn):
         names, _, has_kw, _ = inspect.getargspec(fn)
         return names, bool(has_kw)
 
+
 def get_func_kwargs(func):
     """Return the set of legal kwargs for the given `func`.
 
@@ -118,6 +215,7 @@ def get_func_kwargs(func):
 
     return inspect.getargspec(func)[0]
 
+
 def format_argspec_plus(fn, grouped=True):
     """Returns a dictionary of formatted, introspected function arguments.
 
@@ -150,7 +248,11 @@ def format_argspec_plus(fn, grouped=True):
        'apply_pos': '(self, a, b, c, **d)'}
 
     """
-    spec = callable(fn) and inspect.getargspec(fn) or fn
+    if callable(fn):
+        spec = inspect_getfullargspec(fn)
+    else:
+        # we accept an existing argspec...
+        spec = fn
     args = inspect.formatargspec(*spec)
     if spec[0]:
         self_arg = spec[0][0]
@@ -158,9 +260,31 @@ def format_argspec_plus(fn, grouped=True):
         self_arg = '%s[0]' % spec[1]
     else:
         self_arg = None
+
+    # Py3K
+    #apply_pos = inspect.formatargspec(spec[0], spec[1],
+    #    spec[2], None, spec[4])
+    #num_defaults = 0
+    #if spec[3]:
+    #    num_defaults += len(spec[3])
+    #if spec[4]:
+    #    num_defaults += len(spec[4])
+    #name_args = spec[0] + spec[4]
+    # Py2K
     apply_pos = inspect.formatargspec(spec[0], spec[1], spec[2])
-    defaulted_vals = spec[3] is not None and spec[0][0-len(spec[3]):] or ()
-    apply_kw = inspect.formatargspec(spec[0], spec[1], spec[2], defaulted_vals,
+    num_defaults = 0
+    if spec[3]:
+        num_defaults += len(spec[3])
+    name_args = spec[0]
+    # end Py2K
+
+    if num_defaults:
+        defaulted_vals = name_args[0 - num_defaults:]
+    else:
+        defaulted_vals = ()
+
+    apply_kw = inspect.formatargspec(name_args, spec[1], spec[2],
+                                        defaulted_vals,
                                      formatvalue=lambda x: '=' + x)
     if grouped:
         return dict(args=args, self_arg=self_arg,
@@ -169,6 +293,7 @@ def format_argspec_plus(fn, grouped=True):
         return dict(args=args[1:-1], self_arg=self_arg,
                     apply_pos=apply_pos[1:-1], apply_kw=apply_kw[1:-1])
 
+
 def format_argspec_init(method, grouped=True):
     """format_argspec_plus with considerations for typical __init__ methods
 
@@ -182,7 +307,6 @@ def format_argspec_init(method, grouped=True):
     try:
         return format_argspec_plus(method, grouped=grouped)
     except TypeError:
-        self_arg = 'self'
         if method is object.__init__:
             args = grouped and '(self)' or 'self'
         else:
@@ -190,6 +314,7 @@ def format_argspec_init(method, grouped=True):
                             or 'self, *args, **kwargs')
         return dict(self_arg='self', args=args, apply_pos=args, apply_kw=args)
 
+
 def getargspec_init(method):
     """inspect.getargspec with considerations for typical __init__ methods
 
@@ -209,13 +334,63 @@ def getargspec_init(method):
 
 
 def unbound_method_to_callable(func_or_cls):
-    """Adjust the incoming callable such that a 'self' argument is not required."""
+    """Adjust the incoming callable such that a 'self' argument is not
+    required.
+
+    """
 
     if isinstance(func_or_cls, types.MethodType) and not func_or_cls.im_self:
         return func_or_cls.im_func
     else:
         return func_or_cls
 
+
+def generic_repr(obj, additional_kw=(), to_inspect=None):
+    """Produce a __repr__() based on direct association of the __init__()
+    specification vs. same-named attributes present.
+
+    """
+    if to_inspect is None:
+        to_inspect = obj
+
+    missing = object()
+
+    def genargs():
+        try:
+            (args, vargs, vkw, defaults) = \
+                inspect.getargspec(to_inspect.__init__)
+        except TypeError:
+            return
+
+        default_len = defaults and len(defaults) or 0
+
+        if not default_len:
+            for arg in args[1:]:
+                yield repr(getattr(obj, arg, None))
+            if vargs is not None and hasattr(obj, vargs):
+                yield ', '.join(repr(val) for val in getattr(obj, vargs))
+        else:
+            for arg in args[1:-default_len]:
+                yield repr(getattr(obj, arg, None))
+            for (arg, defval) in zip(args[-default_len:], defaults):
+                try:
+                    val = getattr(obj, arg, missing)
+                    if val is not missing and val != defval:
+                        yield '%s=%r' % (arg, val)
+                except:
+                    pass
+        if additional_kw:
+            for arg, defval in additional_kw:
+                try:
+                    val = getattr(obj, arg, missing)
+                    if val is not missing and val != defval:
+                        yield '%s=%r' % (arg, val)
+                except:
+                    pass
+
+    return "%s(%s)" % (obj.__class__.__name__, ", ".join(genargs()))
+
+
 class portable_instancemethod(object):
     """Turn an instancemethod into a (parent, name) pair
     to produce a serializable callable.
@@ -228,6 +403,7 @@ class portable_instancemethod(object):
     def __call__(self, *arg, **kw):
         return getattr(self.target, self.name)(*arg, **kw)
 
+
 def class_hierarchy(cls):
     """Return an unordered sequence of all classes related to cls.
 
@@ -272,6 +448,7 @@ def class_hierarchy(cls):
             hier.add(s)
     return list(hier)
 
+
 def iterate_attributes(cls):
     """iterate all the keys and attributes associated
        with a class, without using getattr().
@@ -287,6 +464,7 @@ def iterate_attributes(cls):
                 yield (key, c.__dict__[key])
                 break
 
+
 def monkeypatch_proxied_specials(into_cls, from_cls, skip=None, only=None,
                                  name='self.proxy', from_instance=None):
     """Automates delegation of __specials__ for a proxying type."""
@@ -300,6 +478,7 @@ def monkeypatch_proxied_specials(into_cls, from_cls, skip=None, only=None,
         dunders = [m for m in dir(from_cls)
                    if (m.startswith('__') and m.endswith('__') and
                        not hasattr(into_cls, m) and m not in skip)]
+
     for method in dunders:
         try:
             fn = getattr(from_cls, method)
@@ -337,6 +516,7 @@ def methods_equivalent(meth1, meth2):
     return getattr(meth1, 'im_func', meth1) is getattr(meth2, 'im_func', meth2)
     # end Py2K
 
+
 def as_interface(obj, cls=None, methods=None, required=None):
     """Ensure basic interface compliance for an instance or dict of callables.
 
@@ -432,6 +612,9 @@ class memoized_property(object):
         obj.__dict__[self.__name__] = result = self.fget(obj)
         return result
 
+    def _reset(self, obj):
+        obj.__dict__.pop(self.__name__, None)
+
 
 class memoized_instancemethod(object):
     """Decorate a method memoize its return value.
@@ -449,6 +632,7 @@ class memoized_instancemethod(object):
     def __get__(self, obj, cls):
         if obj is None:
             return self
+
         def oneshot(*args, **kw):
             result = self.fget(obj, *args, **kw)
             memo = lambda *a, **kw: result
@@ -456,19 +640,19 @@ class memoized_instancemethod(object):
             memo.__doc__ = self.__doc__
             obj.__dict__[self.__name__] = memo
             return result
+
         oneshot.__name__ = self.__name__
         oneshot.__doc__ = self.__doc__
         return oneshot
 
-def reset_memoized(instance, name):
-    instance.__dict__.pop(name, None)
-
 
 class group_expirable_memoized_property(object):
     """A family of @memoized_properties that can be expired in tandem."""
 
-    def __init__(self):
+    def __init__(self, attributes=()):
         self.attributes = []
+        if attributes:
+            self.attributes.extend(attributes)
 
     def expire_instance(self, instance):
         """Expire all memoized properties for *instance*."""
@@ -480,6 +664,11 @@ class group_expirable_memoized_property(object):
         self.attributes.append(fn.__name__)
         return memoized_property(fn)
 
+    def method(self, fn):
+        self.attributes.append(fn.__name__)
+        return memoized_instancemethod(fn)
+
+
 class importlater(object):
     """Deferred import object.
 
@@ -491,42 +680,75 @@ class importlater(object):
 
         from mypackage.somemodule import somesubmod
 
-    except evaluted upon attribute access to "somesubmod".
+    except evaluated upon attribute access to "somesubmod".
+
+    importlater() currently requires that resolve_all() be
+    called, typically at the bottom of a package's __init__.py.
+    This is so that __import__ still called only at
+    module import time, and not potentially within
+    a non-main thread later on.
 
     """
+
+    _unresolved = set()
+
     def __init__(self, path, addtl=None):
         self._il_path = path
         self._il_addtl = addtl
+        importlater._unresolved.add(self)
+
+    @classmethod
+    def resolve_all(cls):
+        for m in list(importlater._unresolved):
+            m._resolve()
+
+    @property
+    def _full_path(self):
+        if self._il_addtl:
+            return self._il_path + "." + self._il_addtl
+        else:
+            return self._il_path
 
     @memoized_property
     def module(self):
+        if self in importlater._unresolved:
+            raise ImportError(
+                    "importlater.resolve_all() hasn't "
+                    "been called (this is %s %s)"
+                    % (self._il_path, self._il_addtl))
+
+        m = self._initial_import
         if self._il_addtl:
-            m = __import__(self._il_path, globals(), locals(),
-                                [self._il_addtl])
-            try:
-                return getattr(m, self._il_addtl)
-            except AttributeError:
-                raise ImportError(
-                        "Module %s has no attribute '%s'" %
-                        (self._il_path, self._il_addtl)
-                    )
+            m = getattr(m, self._il_addtl)
         else:
-            m = __import__(self._il_path)
             for token in self._il_path.split(".")[1:]:
                 m = getattr(m, token)
-            return m
+        return m
+
+    def _resolve(self):
+        importlater._unresolved.discard(self)
+        if self._il_addtl:
+            self._initial_import = __import__(
+                                self._il_path, globals(), locals(),
+                                [self._il_addtl])
+        else:
+            self._initial_import = __import__(self._il_path)
 
     def __getattr__(self, key):
+        if key == 'module':
+            raise ImportError("Could not resolve module %s"
+                                % self._full_path)
         try:
             attr = getattr(self.module, key)
         except AttributeError:
             raise AttributeError(
                         "Module %s has no attribute '%s'" %
-                        (self._il_path, key)
+                        (self._full_path, key)
                     )
         self.__dict__[key] = attr
         return attr
 
+
 # from paste.deploy.converters
 def asbool(obj):
     if isinstance(obj, (str, unicode)):
@@ -539,6 +761,7 @@ def asbool(obj):
             raise ValueError("String is not true/false: %r" % obj)
     return bool(obj)
 
+
 def bool_or_str(*text):
     """Return a callable that will evaulate a string as
     boolean, or one of a set of "alternate" string values.
@@ -551,6 +774,7 @@ def bool_or_str(*text):
             return asbool(obj)
     return bool_or_value
 
+
 def asint(value):
     """Coerce to integer."""
 
@@ -584,6 +808,23 @@ def constructor_copy(obj, cls, **kw):
     return cls(**kw)
 
 
+def counter():
+    """Return a threadsafe counter function."""
+
+    lock = threading.Lock()
+    counter = itertools.count(1L)
+
+    # avoid the 2to3 "next" transformation...
+    def _next():
+        lock.acquire()
+        try:
+            return counter.next()
+        finally:
+            lock.release()
+
+    return _next
+
+
 def duck_type_collection(specimen, default=None):
     """Given an instance or class, guess if it is or is acting as one of
     the basic collection types: list, set and dict.  If the __emulates__
@@ -593,7 +834,7 @@ def duck_type_collection(specimen, default=None):
     if hasattr(specimen, '__emulates__'):
         # canonicalize set vs sets.Set to a standard: the builtin set
         if (specimen.__emulates__ is not None and
-            issubclass(specimen.__emulates__, set_types)):
+                issubclass(specimen.__emulates__, set_types)):
             return set
         else:
             return specimen.__emulates__
@@ -615,18 +856,19 @@ def duck_type_collection(specimen, default=None):
     else:
         return default
 
+
 def assert_arg_type(arg, argtype, name):
     if isinstance(arg, argtype):
         return arg
     else:
         if isinstance(argtype, tuple):
             raise exc.ArgumentError(
-                            "Argument '%s' is expected to be one of type %s, got '%s'" %
-                            (name, ' or '.join("'%s'" % a for a in argtype), type(arg)))
+                "Argument '%s' is expected to be one of type %s, got '%s'" %
+                (name, ' or '.join("'%s'" % a for a in argtype), type(arg)))
         else:
             raise exc.ArgumentError(
-                            "Argument '%s' is expected to be of type '%s', got '%s'" %
-                            (name, argtype, type(arg)))
+                "Argument '%s' is expected to be of type '%s', got '%s'" %
+                (name, argtype, type(arg)))
 
 
 def dictlike_iteritems(dictlike):
@@ -678,15 +920,36 @@ class classproperty(property):
         return desc.fget(cls)
 
 
-class _symbol(object):
-    def __init__(self, name, doc=None):
+class hybridmethod(object):
+    """Decorate a function as cls- or instance- level."""
+    def __init__(self, func, expr=None):
+        self.func = func
+
+    def __get__(self, instance, owner):
+        if instance is None:
+            return self.func.__get__(owner, owner.__class__)
+        else:
+            return self.func.__get__(instance, owner)
+
+
+class _symbol(int):
+    def __new__(self, name, doc=None, canonical=None):
         """Construct a new named symbol."""
         assert isinstance(name, str)
-        self.name = name
+        if canonical is None:
+            canonical = hash(name)
+        v = int.__new__(_symbol, canonical)
+        v.name = name
         if doc:
-            self.__doc__ = doc
+            v.__doc__ = doc
+        return v
+
     def __reduce__(self):
-        return symbol, (self.name,)
+        return symbol, (self.name, "x", int(self))
+
+    def __str__(self):
+        return repr(self)
+
     def __repr__(self):
         return "<symbol '%s>" % self.name
 
@@ -717,18 +980,20 @@ class symbol(object):
     symbols = {}
     _lock = threading.Lock()
 
-    def __new__(cls, name, doc=None):
+    def __new__(cls, name, doc=None, canonical=None):
         cls._lock.acquire()
         try:
             sym = cls.symbols.get(name)
             if sym is None:
-                cls.symbols[name] = sym = _symbol(name, doc)
+                cls.symbols[name] = sym = _symbol(name, doc, canonical)
             return sym
         finally:
             symbol._lock.release()
 
 
 _creation_order = 1
+
+
 def set_creation_order(instance):
     """Assign a '_creation_order' sequence to the given instance.
 
@@ -739,10 +1004,14 @@ def set_creation_order(instance):
     """
     global _creation_order
     instance._creation_order = _creation_order
-    _creation_order +=1
+    _creation_order += 1
+
 
 def warn_exception(func, *args, **kwargs):
-    """executes the given function, catches all exceptions and converts to a warning."""
+    """executes the given function, catches all exceptions and converts to
+    a warning.
+
+    """
     try:
         return func(*args, **kwargs)
     except:
@@ -755,7 +1024,9 @@ def warn(msg, stacklevel=3):
     If msg is a string, :class:`.exc.SAWarning` is used as
     the category.
 
-    .. note:: This function is swapped out when the test suite
+    .. note::
+
+       This function is swapped out when the test suite
        runs, with a compatible version that uses
        warnings.warn_explicit, so that the warnings registry can
        be controlled.
@@ -766,8 +1037,23 @@ def warn(msg, stacklevel=3):
     else:
         warnings.warn(msg, stacklevel=stacklevel)
 
+
+def only_once(fn):
+    """Decorate the given function to be a no-op after it is called exactly
+    once."""
+
+    once = [fn]
+    def go(*arg, **kw):
+        if once:
+            once_fn = once.pop()
+            return once_fn(*arg, **kw)
+
+    return update_wrapper(go, fn)
+
+
 _SQLA_RE = re.compile(r'sqlalchemy/([a-z_]+/){0,2}[a-z_]+\.py')
 _UNITTEST_RE = re.compile(r'unit(?:2|test2?/)')
+
 def chop_traceback(tb, exclude_prefix=_UNITTEST_RE, exclude_suffix=_SQLA_RE):
     """Chop extraneous lines off beginning and end of a traceback.
 
@@ -786,6 +1072,6 @@ def chop_traceback(tb, exclude_prefix=_UNITTEST_RE, exclude_suffix=_SQLA_RE):
         start += 1
     while start <= end and exclude_suffix.search(tb[end]):
         end -= 1
-    return tb[start:end+1]
+    return tb[start:end + 1]
 
 NoneType = type(None)
